﻿Imports System.ComponentModel

Public Class FilteringBox

    Private _filterTimer As New System.Windows.Forms.Timer()
    Private _fullList As List(Of String)
    Private _worker As FilterWorker

    Public Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        _fullList = New List(Of String)()

        ' Timer needs a container
        If Me.components Is Nothing Then
            Me.components = New Container()
        End If

        _filterTimer = New System.Windows.Forms.Timer(Me.components)
        _filterTimer.Enabled = False
        _filterTimer.Interval = 250
        AddHandler _filterTimer.Tick, AddressOf TimerOnTick

        _worker = New FilterWorker()
        AddHandler _worker.RunWorkerCompleted, AddressOf WorkerOnCompleted
    End Sub

    Public Property SearchText() As String
        Get
            Return SearchTextBox.Text
        End Get
        Set(ByVal value As String)
            SearchTextBox.Text = value
        End Set
    End Property

    Public Property SelectedIndex() As Integer
        Get
            Return ItemsListBox.SelectedIndex
        End Get
        Set(ByVal value As Integer)
            ItemsListBox.SelectedIndex = value
        End Set
    End Property

    Public Property SelectedItem() As String
        Get
            Return ItemsListBox.SelectedItem.ToString
        End Get
        Set(ByVal value As String)
            ItemsListBox.SelectedItem = value
        End Set
    End Property

    Public Sub AddItem(ByVal item As String)
        ItemsListBox.Items.Add(item)
        _fullList.Add(item)
    End Sub

    Public Sub AddItems(ByVal items() As String)
        ItemsListBox.Items.AddRange(items)
        _fullList.AddRange(items)
    End Sub

    Private Sub FilterItems(ByVal searchText As String)
        While _worker.IsBusy
            ' This is kind of ugly
            _worker.CancelAsync()
            Threading.Thread.Sleep(200)
        End While

        _worker.WordList = _fullList.ToArray()
        _worker.FilterText = searchText
        _worker.RunWorkerAsync()
    End Sub

    Private Sub TimerOnTick(ByVal sender As Object, ByVal e As EventArgs)
        ' When the timer ticks, filter the box
        _filterTimer.Stop()
        FilterItems(SearchTextBox.Text)
    End Sub

    Private Sub WorkerOnCompleted(ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs)
        ' Update the box when the filtering is complete
        If Not e.Cancelled Then
            ItemsListBox.BeginUpdate()
            ItemsListBox.Items.Clear()
            Dim items() As String = DirectCast(e.Result, String())
            ItemsListBox.Items.AddRange(items)
            ItemsListBox.SelectedIndex = 0
            ItemsListBox.EndUpdate()
        End If
    End Sub

    Private Sub SearchTextBox_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SearchTextBox.TextChanged
        ' Reset the timer that waits to filter
        _filterTimer.Stop()
        _filterTimer.Start()
    End Sub
End Class
